#!/usr/bin/env python

"""A library that provides a Python interface to the GDELT 1.0 and 2.0 API"""
from __future__ import absolute_import

from gdelt.base import gdelt

__name__ = 'gdelt'
__author__ = 'Linwood Creekmore III'
__email__ = 'valinvescap@gmail.com'
__license__ = 'GNU General Public License v3.0'
__version__ = '0.1.14'
__url__ = 'https://linwoodc3.github.io/gdeltPyR/'
__description__ = 'Python based framework to retreive Global Database of ' \
                  'Events, Language, and Tone (GDELT)' \
                  'data in Python Pandas or R dataframes.'
