Search.setIndex({envversion:49,filenames:["index"],objects:{},objnames:{},objtypes:{},terms:{content:0,index:0,modul:0,page:0,search:0},titles:["Welcome to gdeltPyR&#8217;s documentation!"],titleterms:{document:0,gdeltpyr:0,indice:0,tabl:0,welcom:0}})